import torch
import numpy as np
import torch.nn as nn
import torch.optim as optim
import matplotlib.pyplot as plt
from IPython import display
display.set_matplotlib_formats('svg')
import torch.nn.functional as F
#pip install urllib3==1.26.6
import torchvision
import torchvision.transforms as transforms

#from torchsummary import summary
from torchinfo import summary

IMG_SIZE = 64
BATH_SIZE = 16

#net = torchvision.models.alexnet(weights='DEFAULT')
net = torchvision.models.vgg11_bn(weights='DEFAULT')
#net = torchvision.models.googlenet(weights='DEFAULT')
#net = torchvision.models.resnet18(weights='DEFAULT')
#summary(torch_vgg11, (3, 224, 224))

cnames = ("input_size", "output_size", "num_params", "params_percent", 
          "kernel_size", "mult_adds", "trainable")

cnames = ("output_size", "num_params", "trainable")

print(summary(net, input_size=(1, 3, IMG_SIZE, IMG_SIZE), col_names=cnames, col_width=15))

for param in net.parameters():
    param.requires_grad = False

print(summary(net, input_size=(1, 3, IMG_SIZE, IMG_SIZE), col_names=cnames, col_width=15))

print(net)
net.classifier[6] = nn.Linear(4096, 43)
#net.fc = nn.Linear(1024, 43)
#net.fc = nn.Linear(512, 43)

print(summary(net, input_size=(1, 3, IMG_SIZE, IMG_SIZE), col_names=cnames, col_width=15))

class_names = {
    0: "Speed limit (20km/h)",
    1: "Speed limit (30km/h)",
    2: "Speed limit (50km/h)",
    3: "Speed limit (60km/h)",
    4: "Speed limit (70km/h)",
    5: "Speed limit (80km/h)",
    6: "End of speed limit (80km/h)",
    7: "Speed limit (100km/h)",
    8: "Speed limit (120km/h)",
    9: "No passing",
    10: "No passing veh over 3.5 tons",
    11: "Right-of-way at intersection",
    12: "Priority road",
    13: "Yield",
    14: "Stop",
    15: "No vehicles",
    16: "Veh > 3.5 tons prohibited",
    17: "No entry",
    18: "General caution",
    19: "Dangerous curve left",
    20: "Dangerous curve right",
    21: "Double curve",
    22: "Bumpy road",
    23: "Slippery road",
    24: "Road narrows on the right",
    25: "Road work",
    26: "Traffic signals",
    27: "Pedestrians",
    28: "Children crossing",
    29: "Bicycles crossing",
    30: "Beware of ice/snow",
    31: "Wild animals crossing",
    32: "End speed + passing limits",
    33: "Turn right ahead",
    34: "Turn left ahead",
    35: "Ahead only",
    36: "Go straight or right",
    37: "Go straight or left",
    38: "Keep right",
    39: "Keep left",
    40: "Roundabout mandatory",
    41: "End of no passing",
    42: "End no passing veh > 3.5 tons"
}

transform = transforms.Compose([
    transforms.Resize(IMG_SIZE),
    transforms.CenterCrop(IMG_SIZE),
    transforms.ToTensor(),
    transforms.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]),
])

trainset = torchvision.datasets.GTSRB(root='./data', split="train",
                                        download=True, transform=transform)

testset = torchvision.datasets.GTSRB(root='./data', split="test",
                                       download=True, transform=transform)

testloader = torch.utils.data.DataLoader(testset, batch_size=BATH_SIZE,
                                         shuffle=False, num_workers=1)

trainloader = torch.utils.data.DataLoader(trainset, batch_size=BATH_SIZE,
                                          shuffle=True, num_workers=1)

criterion = nn.CrossEntropyLoss()
#net.classifier.parameters()
#net.fc.parameters()
optimizer = optim.SGD(net.classifier.parameters(), lr=0.001)

for epoch in range(2):  # loop over the dataset multiple times
    
    running_loss = 0.0
    for i, data in enumerate(trainloader, 0):
        # get the inputs; data is a list of [inputs, labels]
        inputs, labels = data

        # zero the parameter gradients
        optimizer.zero_grad()

        # forward + backward + optimize
        outputs = net(inputs)
        loss = criterion(outputs, labels)
        loss.backward()
        optimizer.step()

        # print statistics
        running_loss += loss.item()
        if i % 20 == 19:    # print every 20 mini-batches
            print(f'[{epoch + 1}, {i + 1:5d}] loss: {running_loss / 20:.3f}')
            running_loss = 0.0
            
print('Finished Training')

